import httpx as h
import random
from pathlib import Path
import uuid


class ImageGetException(Exception):
    pass


class ImageGet:
    def __init__(self, base_url, headers, cache_num = 10) -> None:
        self.base_url = base_url
        self.headers = headers
        self.cache_num = cache_num
        self.cache_dir = Path(f"../.cache/{self.__class__.__name__}/")
        self.cache_dir.mkdir(parents=True, exist_ok=True)
        
        self.save_cached_files()


    # Main entrypoint, called externally
    def get_random_image(self) -> bytearray:
        try:
            id = self.get_random_image_ids()[0]
            bytes = self.get_image_file(id)
            self.save_cached_files()
        except (h.ConnectError, h.HTTPStatusError, h.NetworkError, h.RequestError, h.DecodingError, h.TransportError, ImageGetException):
            print("Loading image from cache")
            bytes = self.load_cached_file()

        return bytes


    def save_cached_files(self) -> None:
        """Ensures self.cache_num files are at self.cache_dir at any time"""

        present_count = len(list(self.cache_dir.glob("*")))
        missing = self.cache_num - present_count
        if missing == 0:
            return
    
        ids = self.get_random_image_ids(num = missing)
        for i, id in enumerate(ids):
            print(f"Caching image {i + 1}")
            new_cache = self.cache_dir / f"{uuid.uuid4()}"
            try:
                new_cache.write_bytes(self.get_image_file(id))
            except ImageGetException:
                print("Could not cache image, skipping")
                continue

    def load_cached_file(self) -> bytearray:
        """Returns a random file from self.cache_dir"""
        files = list(self.cache_dir.glob("*"))

        if len(files) == 0:
            raise ImageGetException("Could not load cached file: directory empty")

        file = random.choice(files)
        bytes  = file.read_bytes()
        file.unlink()
        return bytes



    def get_random_image_ids(self, num=1) -> list[str]:
        raise NotImplementedError
    

    def get_image_file(self, image_id: str) -> bytearray:
        raise NotImplementedError
